'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _graphqlCompose = require('graphql-compose');

var _graphql = require('graphql');

var _globalId = require('./globalId');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MutationMiddleware = function (_ResolverMiddleware) {
  (0, _inherits3.default)(MutationMiddleware, _ResolverMiddleware);

  function MutationMiddleware(typeComposer, typeResolver) {
    var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
    (0, _classCallCheck3.default)(this, MutationMiddleware);

    var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(MutationMiddleware).call(this, typeComposer, typeResolver, opts));

    _this.args = function (next) {
      return function (args) {
        var nextArgs = next(args);
        var inputTC = void 0;
        var newNextArgs = {};

        if (nextArgs.input && nextArgs.input.type) {
          // pass args unchanged
          inputTC = new _graphqlCompose.InputTypeComposer((0, _graphql.getNamedType)(nextArgs.input.type));
          newNextArgs = nextArgs;
        } else {
          // create input arg, and put into all current args
          inputTC = new _graphqlCompose.InputTypeComposer(new _graphql.GraphQLInputObjectType({
            name: 'Relay' + _this.typeResolver.getNameCamelCase() + _this.typeComposer.getTypeName() + 'Input',
            fields: nextArgs
          }));
          newNextArgs = {
            input: {
              name: 'input',
              type: new _graphql.GraphQLNonNull(inputTC.getType())
            }
          };
          _this.isArgsWrapped = true;
        }

        // add `clientMutationId` to args.input field
        if (!inputTC.hasField('clientMutationId')) {
          inputTC.addField('clientMutationId', {
            type: _graphql.GraphQLString,
            description: 'The client mutation ID used by clients like Relay to track the mutation. ' + 'If given, returned in the response payload of the mutation.'
          });
        }

        return newNextArgs;
      };
    };

    _this.resolve = function (next) {
      return function (resolveParams) {
        var clientMutationId = void 0;

        if (resolveParams && resolveParams.args) {
          if (resolveParams.args.input && resolveParams.args.input.clientMutationId) {
            clientMutationId = resolveParams.args.input.clientMutationId;
            delete resolveParams.args.input.clientMutationId;
          }
        }

        if (_this.isArgsWrapped) {
          // $FlowFixMe
          resolveParams.args = resolveParams.args.input;
        }

        return next(resolveParams).then(function (res) {
          res.nodeId = (0, _globalId.toGlobalId)(_this.typeComposer.getTypeName(), res.recordId);
          if (clientMutationId) {
            res.clientMutationId = clientMutationId;
          }
          return res;
        });
      };
    };

    _this.outputType = function (next) {
      return function (outputType) {
        var nextOutputType = next(outputType);

        if (!(nextOutputType instanceof _graphql.GraphQLObjectType)) {
          return nextOutputType;
        }

        var outputTC = new _graphqlCompose.TypeComposer(nextOutputType);
        if (!outputTC.hasField('nodeId')) {
          outputTC.addField('nodeId', {
            type: _graphql.GraphQLID,
            description: 'The globally unique ID among all types'
          });
        }

        if (!outputTC.hasField('clientMutationId')) {
          outputTC.addField('clientMutationId', {
            type: _graphql.GraphQLString,
            description: 'The client mutation ID used by clients like Relay to track the mutation. ' + 'If given, returned in the response payload of the mutation.'
          });
        }

        return outputTC.getType();
      };
    };

    _this.isArgsWrapped = false;
    return _this;
  }

  return MutationMiddleware;
}(_graphqlCompose.ResolverMiddleware);
/* eslint-disable no-param-reassign */

exports.default = MutationMiddleware;